@@ -22,13 +22,16 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <gthumb.h>
+#include <extensions/image_viewer/gth-metadata-provider-image.h>
 #include "dlg-resize-images.h"
 #include "preferences.h"
 
 
 #define GET_WIDGET(name) _gtk_builder_get_widget (data->builder, (name))
-#define DEFAULT_WIDTH 640
-#define DEFAULT_HEIGHT 480
+#define DEFAULT_WIDTH_PERCENTAGE 100
+#define DEFAULT_HEIGHT_PERCENTAGE 100
+#define DEFAULT_WIDTH_PIXELS 640
+#define DEFAULT_HEIGHT_PIXELS 480
 
 
 enum {
@@ -47,6 +50,14 @@ typedef struct {
 	GtkBuilder *builder;
 	GtkWidget  *dialog;
 	gboolean    use_destination;
+	gulong      width_spinbutton_event;
+	gulong      height_spinbutton_event;
+	double      latest_width_in_pixel;
+	double      latest_height_in_pixel;
+	double      latest_width_in_percentage;
+	double      latest_height_in_percentage;
+	gboolean    known_ratio;
+	double      ratio;
 } DialogData;
 
 
@@ -182,7 +193,7 @@ ok_clicked_cb (GtkWidget  *widget,
 
 
 static void
-update_sensitivity (DialogData *data)
+update_width_height_properties (DialogData *data)
 {
 	GthUnit unit;
 
@@ -190,13 +201,25 @@ update_sensitivity (DialogData *data)
 	if (unit == GTH_UNIT_PERCENTAGE) {
 		gtk_spin_button_set_digits (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")), 2);
 		gtk_spin_button_set_digits (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")), 2);
+
+		g_signal_handler_block (GET_WIDGET ("height_spinbutton"), data->height_spinbutton_event);
+		g_signal_handler_block (GET_WIDGET ("width_spinbutton"), data->width_spinbutton_event);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")), data->latest_height_in_percentage);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")), data->latest_width_in_percentage);
+		g_signal_handler_unblock (GET_WIDGET ("height_spinbutton"), data->height_spinbutton_event);
+		g_signal_handler_unblock (GET_WIDGET ("width_spinbutton"), data->width_spinbutton_event);
 	}
 	else if (unit == GTH_UNIT_PIXELS) {
 		gtk_spin_button_set_digits (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")), 0);
 		gtk_spin_button_set_digits (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")), 0);
-	}
 
-	gtk_widget_set_sensitive (GET_WIDGET ("keep_ratio_checkbutton"), units[gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")))] != GTH_UNIT_PERCENTAGE);
+		g_signal_handler_block (GET_WIDGET ("height_spinbutton"), data->height_spinbutton_event);
+		g_signal_handler_block (GET_WIDGET ("width_spinbutton"), data->width_spinbutton_event);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")), data->latest_height_in_pixel);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")), data->latest_width_in_pixel);
+		g_signal_handler_unblock (GET_WIDGET ("height_spinbutton"), data->height_spinbutton_event);
+		g_signal_handler_unblock (GET_WIDGET ("width_spinbutton"), data->width_spinbutton_event);
+	}
 }
 
 
@@ -204,7 +227,19 @@ static void
 unit_combobox_changed_cb (GtkComboBox *combobox,
 			  DialogData  *data)
 {
-	update_sensitivity (data);
+	GthUnit unit;
+
+	unit = units[gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")))];
+	if (unit == GTH_UNIT_PERCENTAGE) {
+		data->latest_width_in_pixel = gtk_spin_button_get_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")));
+		data->latest_height_in_pixel = gtk_spin_button_get_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")));
+	}
+	else if (unit == GTH_UNIT_PIXELS) {
+		data->latest_width_in_percentage = gtk_spin_button_get_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")));
+		data->latest_height_in_percentage = gtk_spin_button_get_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")));
+	}
+
+	update_width_height_properties (data);
 }
 
 
@@ -219,6 +254,62 @@ use_destination_checkbutton_toggled_cb (GtkToggleButton *button,
 }
 
 
+static void
+width_spinbutton_value_changed_cb (GtkSpinButton *spinbutton,
+				   gpointer       user_data)
+{
+	DialogData *data = user_data;
+	double      ratio;
+	GthUnit     unit;
+
+	if (! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_ratio_checkbutton"))))
+		return;
+
+	ratio = 0.0;
+	unit = units[gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")))];
+	if ((unit == GTH_UNIT_PIXELS) && data->known_ratio)
+		ratio = 1.0 / data->ratio;
+	else if (unit == GTH_UNIT_PERCENTAGE)
+		ratio = 1.0;
+
+	if (ratio == 0.0)
+		return;
+
+	g_signal_handler_block (GET_WIDGET ("height_spinbutton"), data->height_spinbutton_event);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")),
+				   ratio * gtk_spin_button_get_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton"))));
+	g_signal_handler_unblock (GET_WIDGET ("height_spinbutton"), data->height_spinbutton_event);
+}
+
+
+static void
+height_spinbutton_value_changed_cb (GtkSpinButton *spinbutton,
+				    gpointer       user_data)
+{
+	DialogData *data = user_data;
+	double      ratio;
+	GthUnit     unit;
+
+	if (! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_ratio_checkbutton"))))
+		return;
+
+	ratio = 0.0;
+	unit = units[gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")))];
+	if ((unit == GTH_UNIT_PIXELS) && data->known_ratio)
+		ratio = data->ratio;
+	else if (unit == GTH_UNIT_PERCENTAGE)
+		ratio = 1.0;
+
+	if (ratio == 0.0)
+		return;
+
+	g_signal_handler_block (GET_WIDGET ("width_spinbutton"), data->width_spinbutton_event);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")),
+				   ratio * gtk_spin_button_get_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton"))));
+	g_signal_handler_unblock (GET_WIDGET ("width_spinbutton"), data->width_spinbutton_event);
+}
+
+
 void
 dlg_resize_images (GthBrowser *browser,
 		   GList      *file_list)
@@ -226,6 +317,9 @@ dlg_resize_images (GthBrowser *browser,
 	DialogData  *data;
 	GArray      *savers;
 	GthFileData *first_file_data;
+	GthUnit      unit;
+	int          default_width_in_pixels;
+	int          default_height_in_pixels;
 
 	if (gth_browser_get_dialog (browser, "resize_images") != NULL) {
 		gtk_window_present (GTK_WINDOW (gth_browser_get_dialog (browser, "resize_images")));
@@ -246,11 +340,57 @@ dlg_resize_images (GthBrowser *browser,
 
 	/* Set widgets data. */
 
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")), eel_gconf_get_integer (PREF_RESIZE_IMAGES_SERIES_WIDTH, DEFAULT_WIDTH));
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")), eel_gconf_get_integer (PREF_RESIZE_IMAGES_SERIES_HEIGHT, DEFAULT_HEIGHT));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")), eel_gconf_get_enum (PREF_RESIZE_IMAGES_UNIT, GTH_TYPE_UNIT, GTH_UNIT_PIXELS));
+	first_file_data = (GthFileData *) data->file_list->data;
+	gtk_file_chooser_set_file (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")),
+				   first_file_data->file,
+				   NULL);
+
+	unit = eel_gconf_get_enum (PREF_RESIZE_IMAGES_UNIT, GTH_TYPE_UNIT, GTH_UNIT_PIXELS);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")), unit);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_ratio_checkbutton")), eel_gconf_get_boolean (PREF_RESIZE_IMAGES_KEEP_RATIO, TRUE));
-	update_sensitivity (data);
+
+	default_width_in_pixels = DEFAULT_WIDTH_PIXELS;
+	default_height_in_pixels = DEFAULT_HEIGHT_PIXELS;
+
+	if (data->file_list->next == NULL) {
+		GthMetadataProvider *provider;
+		int                  width;
+		int                  height;
+
+		provider = g_object_new (GTH_TYPE_METADATA_PROVIDER_IMAGE, NULL);
+		gth_metadata_provider_read (provider,
+					    first_file_data,
+					    "image::width,image::height",
+					    NULL);
+		width = g_file_info_get_attribute_int32 (first_file_data->info, "image::width");
+		height = g_file_info_get_attribute_int32 (first_file_data->info, "image::height");
+
+		if ((width > 0) && (height > 0)) {
+			data->known_ratio = TRUE;
+			data->ratio = (double) width / height;
+			default_width_in_pixels = width;
+			default_height_in_pixels = height;
+		}
+
+		g_object_unref (provider);
+	}
+
+	if (unit == GTH_UNIT_PERCENTAGE) {
+		data->latest_width_in_pixel = default_width_in_pixels;
+		data->latest_height_in_pixel = default_height_in_pixels;
+		data->latest_width_in_percentage = eel_gconf_get_integer (PREF_RESIZE_IMAGES_SERIES_WIDTH, DEFAULT_WIDTH_PERCENTAGE);
+		data->latest_height_in_percentage = eel_gconf_get_integer (PREF_RESIZE_IMAGES_SERIES_HEIGHT, DEFAULT_HEIGHT_PERCENTAGE);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")), data->latest_width_in_percentage);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")), data->latest_height_in_percentage);
+	}
+	else if (unit == GTH_UNIT_PIXELS) {
+		data->latest_width_in_percentage = DEFAULT_WIDTH_PERCENTAGE;
+		data->latest_height_in_percentage = DEFAULT_HEIGHT_PERCENTAGE;
+		data->latest_width_in_pixel = eel_gconf_get_integer (PREF_RESIZE_IMAGES_SERIES_WIDTH, default_width_in_pixels);
+		data->latest_height_in_pixel = eel_gconf_get_integer (PREF_RESIZE_IMAGES_SERIES_HEIGHT, default_height_in_pixels);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("width_spinbutton")), data->latest_width_in_pixel);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("height_spinbutton")), data->latest_height_in_pixel);
+	}
 
 	savers = gth_main_get_type_set ("pixbuf-saver");
 	if (savers != NULL) {
@@ -300,11 +440,6 @@ dlg_resize_images (GthBrowser *browser,
 		g_free (default_mime_type);
 	}
 
-	first_file_data = (GthFileData *) data->file_list->data;
-	gtk_file_chooser_set_file (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")),
-				   first_file_data->file,
-				   NULL);
-
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog),
@@ -327,13 +462,25 @@ dlg_resize_images (GthBrowser *browser,
 			  "changed",
 			  G_CALLBACK (unit_combobox_changed_cb),
 			  data);
-        g_signal_connect (GET_WIDGET ("use_destination_checkbutton"),
-                          "toggled",
-                          G_CALLBACK (use_destination_checkbutton_toggled_cb),
-                          data);
+	g_signal_connect (GET_WIDGET ("use_destination_checkbutton"),
+			  "toggled",
+			  G_CALLBACK (use_destination_checkbutton_toggled_cb),
+			  data);
+
+	data->width_spinbutton_event = g_signal_connect (GET_WIDGET ("width_spinbutton"),
+							 "value-changed",
+							 G_CALLBACK (width_spinbutton_value_changed_cb),
+							 data);
+	data->height_spinbutton_event = g_signal_connect (GET_WIDGET ("height_spinbutton"),
+							  "value-changed",
+							  G_CALLBACK (height_spinbutton_value_changed_cb),
+							  data);
 
 	/* Run dialog. */
 
+	update_width_height_properties (data);
+	width_spinbutton_value_changed_cb (NULL, data);
+
         if (GTH_IS_FILE_SOURCE_VFS (gth_browser_get_location_source (browser)))
         	gtk_widget_hide (GET_WIDGET ("use_destination_checkbutton"));
 
