@@ -471,7 +471,6 @@ _cairo_image_surface_transform (cairo_surface_t *source,
 	int              line_start;
 	int              line_step;
 	int              pixel_step;
-	int              destination_stride;
 	unsigned char   *p_source_line;
 	unsigned char   *p_destination_line;
 	unsigned char   *p_source;
@@ -498,7 +497,6 @@ _cairo_image_surface_transform (cairo_surface_t *source,
 
 	destination = cairo_image_surface_create (format, destination_width, destination_height);
 	cairo_surface_flush (destination);
-	destination_stride = cairo_image_surface_get_stride (destination);
 	p_source_line = cairo_image_surface_get_data (source);
 	p_destination_line = cairo_image_surface_get_data (destination) + line_start;
 	while (height-- > 0) {
@@ -590,19 +588,35 @@ _cairo_draw_rounded_box (cairo_t *cr,
 			 double   h,
 			 double   r)
 {
-	cairo_move_to (cr, x, y + r);
-	if (r > 0)
-		cairo_arc (cr, x + r, y + r, r, 1.0 * M_PI, 1.5 * M_PI);
-	cairo_rel_line_to (cr, w - (r * 2), 0);
-	if (r > 0)
-		cairo_arc (cr, x + w - r, y + r, r, 1.5 * M_PI, 2.0 * M_PI);
-	cairo_rel_line_to (cr, 0, h - (r * 2));
-	if (r > 0)
-		cairo_arc (cr, x + w - r, y + h - r, r, 0.0 * M_PI, 0.5 * M_PI);
-	cairo_rel_line_to (cr, - (w - (r * 2)), 0);
-	if (r > 0)
-		cairo_arc (cr, x + r, y + h - r, r, 0.5 * M_PI, 1.0 * M_PI);
-	cairo_rel_line_to (cr, 0, - (h - (r * 2)));
+	if (r == 0) {
+		cairo_rectangle (cr, x, y, w, h);
+	}
+	else if (r == 1) {
+		cairo_move_to (cr, x + 1, y);
+		cairo_rel_line_to (cr, w - 2, 0);
+		cairo_rel_line_to (cr, 1, 1);
+		cairo_rel_line_to (cr, 0, h - 2);
+		cairo_rel_line_to (cr, -1, 1);
+		cairo_rel_line_to (cr, - (w - 2), 0);
+		cairo_rel_line_to (cr, -1, -1);
+		cairo_rel_line_to (cr, 0, - (h - 2));
+		cairo_rel_line_to (cr, 1, -1);
+	}
+	else {
+		cairo_move_to (cr, x, y + r);
+		if (r > 0)
+			cairo_arc (cr, x + r, y + r, r, 1.0 * M_PI, 1.5 * M_PI);
+		cairo_rel_line_to (cr, w - (r * 2), 0);
+		if (r > 0)
+			cairo_arc (cr, x + w - r, y + r, r, 1.5 * M_PI, 2.0 * M_PI);
+		cairo_rel_line_to (cr, 0, h - (r * 2));
+		if (r > 0)
+			cairo_arc (cr, x + w - r, y + h - r, r, 0.0 * M_PI, 0.5 * M_PI);
+		cairo_rel_line_to (cr, - (w - (r * 2)), 0);
+		if (r > 0)
+			cairo_arc (cr, x + r, y + h - r, r, 0.5 * M_PI, 1.0 * M_PI);
+		cairo_rel_line_to (cr, 0, - (h - (r * 2)));
+	}
 }
 
 
