@@ -50,6 +50,7 @@ typedef struct {
 
 	/*< private >*/
 
+	guint        ref_count;
 	guint        pos;
 	guint        abs_pos;
 	gboolean     visible;
@@ -81,7 +82,12 @@ static GObjectClass *parent_class = NULL;
 static GthFileRow *
 _gth_file_row_new (void)
 {
-	return g_new0 (GthFileRow, 1);
+	GthFileRow *row;
+
+	row = g_new0 (GthFileRow, 1);
+	row->ref_count = 1;
+
+	return row;
 }
 
 
@@ -146,9 +152,20 @@ _gth_file_row_copy (GthFileRow *row)
 }
 
 
-static void
-_gth_file_row_free (GthFileRow *row)
+static GthFileRow *
+_gth_file_row_ref (GthFileRow *row)
 {
+	row->ref_count++;
+	return row;
+}
+
+
+static void
+_gth_file_row_unref (GthFileRow *row)
+{
+	if (--row->ref_count > 0)
+		return;
+
 	if (row->file_data != NULL)
 		g_object_unref (row->file_data);
 	if (row->thumbnail != NULL)
@@ -161,6 +178,7 @@ _gth_file_row_free (GthFileRow *row)
 static void
 _gth_file_store_clear_queue (GthFileStore *file_store)
 {
+	g_list_foreach (file_store->priv->queue, (GFunc) _gth_file_row_unref, NULL);
 	g_list_free (file_store->priv->queue);
 	file_store->priv->queue = NULL;
 }
@@ -172,7 +190,7 @@ _gth_file_store_free_rows (GthFileStore *file_store)
 	int i;
 
 	for (i = 0; i < file_store->priv->tot_rows; i++)
-		_gth_file_row_free (file_store->priv->all_rows[i]);
+		_gth_file_row_unref (file_store->priv->all_rows[i]);
 	g_free (file_store->priv->all_rows);
 	file_store->priv->all_rows = NULL;
 	file_store->priv->tot_rows = 0;
@@ -800,7 +818,8 @@ _gth_file_store_hide_row (GthFileStore *file_store,
 
 static void
 _gth_file_store_update_visibility (GthFileStore *file_store,
-				   GList        *add_queue)
+				   GList        *add_queue,
+				   int           position)
 {
 	GthFileRow **all_rows = NULL;
 	guint        all_rows_n = 0;
@@ -814,7 +833,6 @@ _gth_file_store_update_visibility (GthFileStore *file_store,
 	GthFileData *file;
 	int          j, k;
 	gboolean     row_deleted;
-	gboolean     row_inserted;
 
 
 #ifdef DEBUG_FILE_STORE
@@ -825,11 +843,21 @@ g_print ("UPDATE VISIBILITY\n");
 
 	all_rows_n = file_store->priv->tot_rows + g_list_length (add_queue);
 	all_rows = g_new (GthFileRow *, all_rows_n);
-	for (i = 0; i < file_store->priv->tot_rows; i++)
-		all_rows[i] = _gth_file_row_copy (file_store->priv->all_rows[i]);
 
+	/* append to the end if position is -1 */
+
+	if (position == -1)
+		position = file_store->priv->tot_rows;
+
+	/* insert the new rows at position */
+
+	j = 0;
+	for (i = 0; i < position; i++)
+		all_rows[j++] = _gth_file_row_copy (file_store->priv->all_rows[i]);
 	for (scan = add_queue; scan; scan = scan->next)
-		all_rows[i++] = (GthFileRow *) scan->data;
+		all_rows[j++] = _gth_file_row_ref ((GthFileRow *) scan->data);
+	for (i = position; i < file_store->priv->tot_rows; i++)
+		all_rows[j++] = _gth_file_row_copy (file_store->priv->all_rows[i]);
 
 	old_rows_n = file_store->priv->num_rows;
 	old_rows = g_new (GthFileRow *, old_rows_n);
@@ -997,7 +1025,7 @@ g_print ("\n");
 	_gth_file_store_increment_stamp (file_store);
 
 	for (i = 0; i < file_store->priv->tot_rows; i++)
-		_gth_file_row_free (file_store->priv->all_rows[i]);
+		_gth_file_row_unref (file_store->priv->all_rows[i]);
 	g_free (file_store->priv->all_rows);
 	file_store->priv->all_rows = all_rows;
 	file_store->priv->tot_rows = all_rows_n;
@@ -1017,7 +1045,6 @@ g_print ("\n");
 
 	/* add the new files */
 
-	row_inserted = FALSE;
 	for (i = 0; i < new_rows_n; i++) {
 		GtkTreePath *path;
 		GtkTreeIter  iter;
@@ -1044,8 +1071,6 @@ g_print ("  INSERT: %d\n", i);
 		gth_file_store_get_iter (GTK_TREE_MODEL (file_store), &iter, path);
 		gtk_tree_model_row_inserted (GTK_TREE_MODEL (file_store), path, &iter);
 		gtk_tree_path_free (path);
-
-		row_inserted = TRUE;
 	}
 
 	g_signal_emit (file_store, gth_file_store_signals[VISIBILITY_CHANGED], 0);
@@ -1070,7 +1095,7 @@ gth_file_store_set_filter (GthFileStore *file_store,
 	else
 		file_store->priv->filter = gth_test_new ();
 
-	_gth_file_store_update_visibility (file_store, NULL);
+	_gth_file_store_update_visibility (file_store, NULL, -1);
 }
 
 
@@ -1397,10 +1422,11 @@ gth_file_store_add (GthFileStore *file_store,
 		    GdkPixbuf    *thumbnail,
 		    gboolean      is_icon,
 		    const char   *metadata,
-		    gboolean      checked)
+		    gboolean      checked,
+		    int           position)
 {
 	gth_file_store_queue_add (file_store, file, thumbnail, is_icon, metadata, checked);
-	gth_file_store_exec_add (file_store);
+	gth_file_store_exec_add (file_store, position);
 }
 
 
@@ -1428,9 +1454,10 @@ gth_file_store_queue_add (GthFileStore *file_store,
 
 
 void
-gth_file_store_exec_add (GthFileStore *file_store)
+gth_file_store_exec_add (GthFileStore *file_store,
+			 int           position)
 {
-	_gth_file_store_update_visibility (file_store, file_store->priv->queue);
+	_gth_file_store_update_visibility (file_store, file_store->priv->queue, position);
 	_gth_file_store_clear_queue (file_store);
 }
 
@@ -1542,7 +1569,7 @@ gth_file_store_exec_set (GthFileStore *file_store)
 	_gth_file_store_list_changed (file_store);
 	_gth_file_store_clear_queue (file_store);
 	if (file_store->priv->update_filter) {
-		_gth_file_store_update_visibility (file_store, NULL);
+		_gth_file_store_update_visibility (file_store, NULL, -1);
 		file_store->priv->update_filter = FALSE;
 	}
 	else if (file_store->priv->check_changed) {
@@ -1571,7 +1598,7 @@ gth_file_store_queue_remove (GthFileStore *file_store,
 
   	row = (GthFileRow*) iter->user_data;
 
-	file_store->priv->queue = g_list_prepend (file_store->priv->queue, file_store->priv->all_rows[row->abs_pos]);
+	file_store->priv->queue = g_list_prepend (file_store->priv->queue, _gth_file_row_ref (file_store->priv->all_rows[row->abs_pos]));
 }
 
 
@@ -1591,12 +1618,12 @@ gth_file_store_exec_remove (GthFileStore *file_store)
 			_gth_file_store_hide_row (file_store, row);
 
 		file_store->priv->all_rows[row->abs_pos] = NULL;
-		_gth_file_row_free (row);
+		_gth_file_row_unref (row);
 	}
 	_gth_file_store_compact_rows (file_store);
 	_gth_file_store_clear_queue (file_store);
 
-	_gth_file_store_update_visibility (file_store, NULL);
+	_gth_file_store_update_visibility (file_store, NULL, -1);
 }
 
 
